#include "i2c.h"

#define F_CPU 16000000UL
#define F_SCL 100000UL

#define I2C_READ    1
#define I2C_WRITE    0

#define SCL_CLOCK ((F_CPU/F_SCL)-16)/2

void i2c_init(void)
{
    TWBR = (uint8_t)SCL_CLOCK;
}

void i2c_start(void)
{
    // transmit START condition
    TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);
    // wait for end of transmission
    while (!(TWCR & (1<<TWINT)));
}

void i2c_stop(void)
{
    // transmit STOP condition
    TWCR = (1<<TWINT) | (1<<TWSTO) | (1<<TWEN);
    // wait for end of transmission
    while ((TWCR & (1<<TWSTO)));
}

void i2c_write_byte(uint8_t byte)
{
    // transmit data
    TWDR = byte;
    TWCR = (1<<TWINT) | (1<<TWEN);
    // wait for end of transmission
    while (!(TWCR & (1<<TWINT)));
}

uint8_t i2c_read_byte(uint8_t ack)
{
    // receive data
    TWCR = (1<<TWINT) | (1<<TWEN) | (ack<<TWEA);
    // wait for end of transmission
    while (!(TWCR & (1<<TWINT)));
    return TWDR;
}